VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ExcelChemFormatter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' Copyright 2003-2004 by Kazuya Ujihara <ujihara.jp>.
'
' The contents of this file are subject to the Mozilla Public License Version 1.1
' (the "License"); you may not use this file except in compliance with the License.
' You may obtain a copy of the License at http://www.mozilla.org/MPL/
'
' Software distributed under the License is distributed on an "AS IS" basis,
' WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
' for the specific language governing rights and limitations under the License.
'
' The Original Code is 'ChemFormatter'.
'
' The Initial Developer of the Original Code is Kazuya Ujihara. Portions created by
' the Initial Developer are Copyright (C) 2003-2004 by Kazuya Ujihara.
' All Rights Reserved.
'
' Contributor(s): all the names of the contributors are added in the source code
' where applicable.

Option Explicit

Private IonQuery As IonFormulaQuery
Private CNQuery As ChemNameQuery
Private CFQuery As ChemFormulaQuery
Private JQuery As JournalQuery
Private RSChangerQuery As ChemRDigitQuery

'********************************************************************************

Public Application As Excel.Application

Dim m_CommandBuilder As Object
Dim m_Runner As CommandRunner
Dim m_ApplyIt As Boolean

Dim m_Cell As Object ' Excel.Range
Dim CursorPos As Long
Dim CursorLength As Long
Dim ClipedText As String
Const ClipBoardEmpty = "###EMPTY###"

Public Sub GetSubscript(ByVal Index As Long, ByVal Length As Long, ByRef v As Long)   'v: false=>0, true=> -1, mixture=>9999999
    v = m_Cell.Characters(Index + 1, 1).Font.Subscript
End Sub
Public Sub GetSuperscript(ByVal Index As Long, ByVal Length As Long, ByRef v As Long)   'v: false=>0, true=> -1, mixture=>9999999
    v = m_Cell.Characters(Index + 1, 1).Font.Superscript
End Sub

'returns it did something or not.
Private Function ApplyIt(ByVal CommandBuilder As Object, Optional ByVal RecoverSelection As Boolean = True) As Boolean
    m_ApplyIt = False
    
    Set m_Runner = New CommandRunner
    Set m_Runner.Executer = Me
    
    Set m_CommandBuilder = CommandBuilder
    
On Error GoTo Event_End
    Dim Cells As Object
    Set Cells = Application.Selection
On Error GoTo Event_Err
    Call FormatThem(Cells)
On Error GoTo 0
Event_End:
    
    Set m_CommandBuilder = Nothing
    Set m_Runner = Nothing
    
    ApplyIt = m_ApplyIt
    Exit Function

Event_Err:
    AddInErr Err
    Resume Event_End
End Function

Private Sub FormatThem(ByVal sel As Object)
    If IsCollection(sel) Then
        If TypeOf sel Is Excel.Range Then
            Dim Range As Excel.Range
            Set Range = sel
            Select Case Range.Count
            Case 0
                Exit Sub
            Case 1
                Call FormatIt(sel)
                Exit Sub
            Case Is > 1000
                If vbCancel = MsgBox("Too many cells are selected. It may take a long time. Are you sure to apply them ?", vbOKCancel, "ChemFormatter for Excel") Then Exit Sub
            Case Else
                'do nothing
            End Select
        End If

        Dim o As Object
        For Each o In sel
            Call FormatThem(o)
        Next
    Else
        Call FormatIt(sel)
    End If
End Sub

Private Sub FormatIt(ByVal o As Object)
    Dim Chart As Excel.Chart

    If o Is Nothing Then
        Exit Sub
    ElseIf TypeOf o Is Excel.Chart Then
        Set Chart = o
        If Chart.HasTitle Then Call DoFormat(Chart.ChartTitle)
        Dim Axis As Excel.Axis
        For Each Axis In Chart.Axes
            Call DoFormat(Axis.AxisTitle)
        Next
    Else
        Set Chart = GetChart(o)
        If Chart Is Nothing Then
            Call DoFormat(o)
        ElseIf TypeOf Chart Is Excel.Chart Then
            Call FormatIt(Chart)
        Else
            Err.Raise 1, , "Assertion failed in ChemFFExcel."
        End If
    End If
End Sub

Private Function GetChart(ByVal o As Object) As Excel.Chart
    Set GetChart = Nothing
    On Error Resume Next
    Set GetChart = o.Chart
    If Not GetChart Is Nothing Then If TypeOf GetChart Is Excel.Chart Then Exit Function
    Set GetChart = o.Parent
    If Not GetChart Is Nothing Then If TypeOf GetChart Is Excel.Chart Then Exit Function
    On Error GoTo 0
End Function

Private Sub DoFormat(ByVal o As Object)
    Dim Commands As String
    Set m_Cell = o
    Commands = m_CommandBuilder.GetCommands(o.Text)
    Commands = XTrim(Commands)
    If Commands <> "" Then
        CursorPos = 0
        CursorLength = 0
        ClipedText = ClipBoardEmpty
        
        Call m_Runner.Run(Commands)
        m_ApplyIt = True
    End If
End Sub

Public Sub DeleteText(ByVal Index As Long, ByVal Length As Long)
    Dim SI As Long, SL As Long
    SI = Index: SL = Length
    If Index < 0 Then
        SI = 0: SL = SL + Index
    End If
    If SL > 0 Then
        m_Cell.Characters(SI + 1, SL).Delete
    End If
    CursorPos = Index
    CursorLength = 0
End Sub

Public Sub TypeBackspace()
    Call DeleteText(CursorPos - 1, 1)
End Sub

Public Sub TypeParagraph()
    Call TypeText(vbCr)
End Sub

Public Sub PasteText()
    Call DeleteText(CursorPos, CursorLength)
    If ClipedText <> ClipBoardEmpty Then
        Call TypeText(ClipedText)
    End If
End Sub

Public Sub CopyText(ByVal Index As Long, ByVal Length As Long)
    ClipedText = m_Cell.Characters(Index + 1, Length).Text
End Sub

Public Sub CutText(ByVal Index As Long, ByVal Length As Long)
    Call CopyText(Index, Length)
    Call DeleteText(Index, Length)
End Sub

Public Function QueryValue(ByVal x As String) As String
    Select Case x
    Case "$CursorPosition"
        QueryValue = CursorPos
    Case Else
        QueryValue = x
    End Select
End Function

Public Sub MoveCursor(ByVal s As String)
    Dim p As Long
    On Error GoTo 600
    p = s
    GoTo 500
600:
    On Error GoTo 0
    Select Case s
    Case "StartOfSelection"
        p = CursorPos
    Case "EndOfSelection"
        CursorPos = CursorPos + CursorLength
        CursorLength = 0
    Case "PrevChar"
        CursorPos = CursorPos - 1
        CursorLength = 0
    Case "NextChar"
        CursorPos = CursorPos + 1
        CursorLength = 0
    Case Else
        '
    End Select
500:
End Sub

Public Sub TypeText(ByVal s As String, Optional ByVal StyleName As String = vbNullString)
    If Len(s) <= 0 Then Exit Sub
    Call DeleteText(CursorPos, CursorLength)
    Call m_Cell.Characters(CursorPos + 1, CursorLength).Insert(s)
    Call SetStyle(CursorPos, Len(s), StyleName)
    CursorPos = CursorPos + Len(s)
    CursorLength = 0
End Sub

Public Sub ReplaceChar(ByVal Index As Long, ByVal CharCode As Integer)
    Call DeleteText(Index, 1)
    Call TypeText(ChrW$(CharCode))
End Sub

Public Sub SetSubscript(ByVal Index As Long, ByVal Length As Long, Optional ByVal v As Long = msoTrue)
    If Length = 0 Then Exit Sub
    m_Cell.Characters(Index + 1, Length).Font.Subscript = v
End Sub
Public Sub SetSuperscript(ByVal Index As Long, ByVal Length As Long, Optional ByVal v As Long = msoTrue)
    If Length = 0 Then Exit Sub
    m_Cell.Characters(Index + 1, Length).Font.Superscript = v
End Sub

Public Sub SetSmall(ByVal Index As Long, ByVal Length As Long, Optional ByVal v As Long = msoTrue)
    If Length = 0 Then Exit Sub
    Call SetStyle(Index, Length, "Small")
End Sub

Public Sub SelectRange(ByVal Index As Long, ByVal Length As Long)
    CursorPos = Index
    CursorLength = Length
End Sub

Public Sub SetStyle(ByVal Index As Long, ByVal Length As Long, ByVal StyleName As String)
    If Length = 0 Then Exit Sub
    Dim Font As Excel.Font
    Set Font = m_Cell.Characters(Index + 1, Length).Font
    Select Case StyleName
    Case vbNullString
        Font.Italic = False
        Font.Bold = False
        Font.Subscript = False
        Font.Superscript = False
    Case "Italic", "Chemical Italic"
        Font.Italic = True
    Case "Bold"
        Font.Bold = True
    Case "Small", "Chemical Small"
        If Not IsNull(m_Cell.Font.Size) Then
            Font.Size = m_Cell.Font.Size * 0.8
        End If
    Case Else
        '
    End Select
End Sub

Public Sub ApplyStyle(ByVal StyleName As String)
    Call SetStyle(CursorPos, CursorLength, StyleName)
End Sub

'********************************************************************************
'
Public Sub ApplyIon()
    If IonQuery Is Nothing Then
        Set IonQuery = New IonFormulaQuery
    End If
    Call ApplyIt(IonQuery)
End Sub

Public Sub ApplyChemName()
    If CNQuery Is Nothing Then
        Set CNQuery = New ChemNameQuery
        CNQuery.ItalicCommand = "Chemical Italic"
        CNQuery.SmallCommand = "Small"
    End If
    Call ApplyIt(CNQuery)
End Sub

Public Sub ApplyChemFormula()
    If CFQuery Is Nothing Then
        Set CFQuery = New ChemFormulaQuery
        CFQuery.ItalicCommand = "Chemical Italic"
    End If
    Call ApplyIt(CFQuery)
    
End Sub

Public Sub ApplyJournalRef()
    If JQuery Is Nothing Then Set JQuery = New JournalQuery
    Call ApplyIt(JQuery)
End Sub

Sub ApplyRSChanger()
    If RSChangerQuery Is Nothing Then
        Set RSChangerQuery = New ChemRDigitQuery
        Set RSChangerQuery.FontListener = Me
    End If
    Call ApplyIt(RSChangerQuery)
End Sub

Function FormatAsNMRSpec()
    Dim i As Long, j As Long

    Dim o As NMRSpecFormQuery
    Set o = New NMRSpecFormQuery
    o.SpecFormat = GetDocProperty(NMRSpecFormatPropName, NMRSpecFormatDefault)
    o.JValueStyle = GetDocProperty(JValuesFormatPropName, JValuesFormaDefault)
    
    Dim Text As String
    Dim OX As Long, OY As Long
    OX = Application.Selection.Column
    OY = Application.Selection.Row
    Dim HABA As Long, KAZU As Long
    HABA = Application.Selection.Columns.Count
    If HABA > 5 Then HABA = 5
    KAZU = Application.Selection.Rows.Count

    Text = ""
    For i = 1 To KAZU
        For j = 1 To HABA
            Text = Text + Application.Cells(OY + i - 1, OX + j - 1).Text
            If j <> HABA Then Text = Text + vbTab
        Next
        Text = Text + vbCr
    Next
    
    Dim Commands As String
    Commands = XTrim(o.GetCommands(Text))

    Dim Runner As CommandRunner
    Set Runner = New CommandRunner
    Set Runner.Executer = Me
    
    If Commands <> "" Then
        Set m_Cell = Application.Cells(OY, OX)
        m_Cell = "'" & Text
        CursorPos = 0
        CursorLength = Len(Text)
        ClipedText = ClipBoardEmpty
        Call Runner.Run(Commands)
    End If
End Function

Public Sub ShowNMRSpecSetting()
    Dim frmw As frmNMRSpecFormat
    Set frmw = New frmNMRSpecFormat
    
    frmw.NMRSpecFormat = GetDocProperty(NMRSpecFormatPropName, NMRSpecFormatDefault)
    frmw.JValuesStyle = GetDocProperty(JValuesFormatPropName, JValuesFormaDefault)
    frmw.NMRTableFormat = GetDocProperty(NMRTableFormatPropName, frmw.NMRSpecFormat)
    
#If VBA = 0 Then
    frmw.Show vbModal
#Else
    frmw.Show
#End If
    
    If frmw.DialogResult = vbOK Then
        Call SetDocProperty(NMRSpecFormatPropName, frmw.NMRSpecFormat)
        Call SetDocProperty(JValuesFormatPropName, frmw.JValuesStyle)
        Call SetDocProperty(NMRTableFormatPropName, frmw.NMRTableFormat)
    End If
End Sub

Public Function GetDocProperty(ByVal PropName As String, ByVal Default As String) As String
    Dim cps As Object 'Office.DocumentProperties
    Set cps = Application.ActiveWorkbook.CustomDocumentProperties
    GetDocProperty = Default
    On Error Resume Next
    GetDocProperty = cps.Item(PropName).Value
End Function

Public Sub SetDocProperty(ByVal PropName As String, ByVal Value As String)
    Dim cps As Object 'Office.DocumentProperties
    Set cps = Application.ActiveWorkbook.CustomDocumentProperties
    On Error GoTo 1000
    cps.Item(PropName).Value = Value
2000:
    On Error GoTo 0
    Exit Sub
1000:
    Call cps.Add(PropName, False, msoPropertyTypeString, Value)
    Resume 2000
End Sub

Public Function Supported(ByVal Command As String) As Boolean
    Select Case Command
    Case _
        "ChangeStyleForR", _
        "ApplyStyleForChemName", _
        "ApplyStyleForChemFormula", _
        "ApplyStyleForJournalReference", _
        "ApplyStyleForIon"
        Supported = True
    Case _
        "ApplyNMRFormat", _
        "ShowNMRFormatSetting"
        Supported = False
    Case Else
        Supported = False
    End Select
End Function



