VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PublisherChemFormatter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' Copyright 2003-2004 by Kazuya Ujihara <ujihara.jp>.
'
' The contents of this file are subject to the Mozilla Public License Version 1.1
' (the "License"); you may not use this file except in compliance with the License.
' You may obtain a copy of the License at http://www.mozilla.org/MPL/
'
' Software distributed under the License is distributed on an "AS IS" basis,
' WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
' for the specific language governing rights and limitations under the License.
'
' The Original Code is 'ChemFormatter'.
'
' The Initial Developer of the Original Code is Kazuya Ujihara. Portions created by
' the Initial Developer are Copyright (C) 2003-2004 by Kazuya Ujihara.
' All Rights Reserved.
'
' Contributor(s): all the names of the contributors are added in the source code
' where applicable.

Option Explicit

Private IonQuery As IonFormulaQuery
Private CNQuery As ChemNameQuery
Private CFQuery As ChemFormulaQuery
Private JQuery As JournalQuery
Private RSChangerQuery As ChemRDigitQuery

'********************************************************************************

Dim m_CommandBuilder As Object
Dim m_Runner As CommandRunner
Dim m_ApplyIt As Boolean

Public Application As Publisher.Application
Dim l_TextRange As Publisher.TextRange
Dim CursorPos As Long
Dim CursorLength As Long

Public Sub GetSubscript(ByVal Index As Long, ByVal Length As Long, ByRef v As Long)   'v: false=>0, true=> -1, mixture=>9999999
    v = GetTextRange(Index, Length).Font.Subscript
End Sub
Public Sub GetSuperscript(ByVal Index As Long, ByVal Length As Long, ByRef v As Long)   'v: false=>0, true=> -1, mixture=>9999999
    v = GetTextRange(Index, Length).Font.Superscript
End Sub

'returns it did something or not.
Private Function ApplyIt(ByVal CommandBuilder As Object, Optional ByVal RecoverSelection As Boolean = True) As Boolean
    ApplyIt = False
    
    Set m_CommandBuilder = CommandBuilder
    Set m_Runner = New CommandRunner
    Set m_Runner.Executer = Me
    
    m_ApplyIt = False
    
On Error GoTo Event_Err

    Dim sel As Publisher.Selection
    Set sel = Application.Selection
    Select Case sel.Type
    Case pbSelectionNone
        'do nothing
    Case pbSelectionShape
        Dim Shape As Publisher.Shape
        For Each Shape In sel.ShapeRange
            Select Case Shape.Type
            Case pbTable
                Dim Table As Publisher.Table
                Set Table = Shape.Table
                Call ApplyCellRange(Table.Cells)
            Case pbTextFrame
                Dim TextFrame As Publisher.TextFrame
                Set TextFrame = Shape.TextFrame
                Call ApplyTextRange(TextFrame.TextRange)
            Case Else
                'do nothing
            End Select
        Next
    Case pbSelectionText
        Call ApplyTextRange(sel.TextRange)
    Case pbSelectionTableCells
        Call ApplyCellRange(sel.TableCellRange)
    Case pbSelectionShapeSubSelection
        'do nothing
    Case Else
        'do nothing
    End Select

On Error GoTo Event_End

On Error GoTo 0
Event_End:
    
    Set m_Runner = Nothing
    ApplyIt = m_ApplyIt
    Exit Function

Event_Err:
    AddInErr Err
    Resume Event_End
End Function

Private Sub ApplyTextRange(ByVal TextRange As Publisher.TextRange)
    Set l_TextRange = TextRange.Duplicate
    CursorPos = 0
    CursorLength = Len(l_TextRange.Text)

    Dim qq As String
    qq = m_CommandBuilder.GetCommands(l_TextRange.Text)
    
    Dim Commands As String
    Commands = XTrim(qq)
    If Commands <> "" Then
        Call m_Runner.Run(Commands)
        m_ApplyIt = True
    End If
End Sub

Private Sub ApplyCellRange(ByVal CellRange As Publisher.CellRange)
    Dim Cell As Publisher.Cell
    For Each Cell In CellRange
        Call ApplyTextRange(Cell.TextRange)
    Next
End Sub

Public Sub DeleteText(ByVal Index As Long, ByVal Length As Long)
    GetTextRange(Index, Length).Text = ""
    Call SelectRange(Index, 0)
End Sub

Public Sub TypeBackspace()
    GetTextRange(CursorPos - 1, 1).Text = ""
    Call SelectRange(CursorPos - 1, 0)
End Sub

Public Sub TypeParagraph()
    GetTextRange(CursorPos, 0).Text = vbCr
    Call SelectRange(CursorPos + 1, 0)
End Sub

Public Sub PasteText()
    Dim l As Long
    l = GetTextRange(CursorPos, 0).Paste.Length
    Call SelectRange(CursorPos, 0)
End Sub

Public Sub CopyText(ByVal Index As Long, ByVal Length As Long)
    GetTextRange(Index, Length).Copy
    Call SelectRange(Index, Length)
End Sub

Public Sub CutText(ByVal Index As Long, ByVal Length As Long)
    GetTextRange(Index, Length).Cut
    Call SelectRange(Index, 0)
End Sub

Public Function QueryValue(ByVal x As String) As String
    Select Case x
    Case "$CursorPosition"
        QueryValue = CursorPos
    Case Else
        QueryValue = x
    End Select
End Function

Public Sub MoveCursor(ByVal s As String)
    Dim p As Long
    On Error GoTo 600
    p = s
    GoTo 500
600:
    On Error GoTo 0
    Select Case s
    Case "StartOfSelection"
        p = CursorPos
    Case "EndOfSelection"
        p = CursorPos + CursorLength
    Case "PrevChar"
        p = CursorPos - 1
    Case "NextChar"
        p = CursorPos + 1
    Case Else
        '
    End Select
500:
    Call SelectRange(p, 0)
End Sub

Public Sub TypeText(ByVal s As String, Optional ByVal StyleName As String = vbNullString)
    Dim l As Long
    l = Len(s)
    If l <= 0 Then Exit Sub
    Dim Range As Publisher.TextRange
    Set Range = GetTextRange(CursorPos, CursorLength)
    Range.Text = s
    Call SetStyle(CursorPos, l, StyleName)
    Call SelectRange(CursorPos + l, 0)
End Sub

Public Sub ReplaceChar(ByVal Index As Long, ByVal CharCode As Integer)
    GetTextRange(Index, 1).Text = ChrW$(CharCode)
End Sub

Public Sub SetSubscript(ByVal Index As Long, ByVal Length As Long, Optional ByVal v As Long = msoTrue)
    If Length = 0 Then Exit Sub
    GetTextRange(Index, Length).Font.Subscript = v
End Sub
Public Sub SetSuperscript(ByVal Index As Long, ByVal Length As Long, Optional ByVal v As Long = msoTrue)
    If Length = 0 Then Exit Sub
    GetTextRange(Index, Length).Font.Superscript = v
End Sub

Public Sub SetSmall(ByVal Index As Long, ByVal Length As Long, Optional ByVal v As Long = msoTrue)
    If Length = 0 Then Exit Sub
    Call SetStyle(Index, Length, "Small")
End Sub

Public Sub SelectRange(ByVal Index As Long, ByVal Length As Long)
    CursorPos = Index
    CursorLength = Length
End Sub

Private Function GetTextRange(ByVal Index As Long, ByVal Length As Long) As Publisher.TextRange
    Set GetTextRange = l_TextRange.Characters(Index + 1, Length)
End Function

Public Sub SetStyle(ByVal Index As Long, ByVal Length As Long, ByVal StyleName As String)
    If Length = 0 Then Exit Sub
    Dim Font As Publisher.Font
    Set Font = GetTextRange(Index, Length).Font
    Select Case StyleName
    Case vbNullString
        Font.Italic = False
        Font.Bold = False
        Font.Subscript = False
        Font.Superscript = False
    Case "Italic", "Chemical Italic"
        Font.Italic = True
    Case "Bold"
        Font.Bold = True
    Case "Small", "Chemical Small"
        If IsNotMixedSize(l_TextRange.Font.Size) Then
            Font.Size = Font.Size * 0.8
        End If
    Case Else
        '
    End Select
End Sub

Public Sub ApplyStyle(ByVal StyleName As String)
    Call SetStyle(CursorPos, CursorLength, StyleName)
End Sub

'********************************************************************************
'
Public Sub ApplyIon()
    If IonQuery Is Nothing Then
        Set IonQuery = New IonFormulaQuery
    End If
    Call ApplyIt(IonQuery)
End Sub

Public Sub ApplyChemName()
    If CNQuery Is Nothing Then
        Set CNQuery = New ChemNameQuery
        CNQuery.ItalicCommand = "Chemical Italic"
        CNQuery.SmallCommand = "Small"
    End If
    Call ApplyIt(CNQuery)
End Sub

Public Sub ApplyChemFormula()
    If CFQuery Is Nothing Then
        Set CFQuery = New ChemFormulaQuery
        CFQuery.ItalicCommand = "Chemical Italic"
    End If
    Call ApplyIt(CFQuery)
    
End Sub

Public Sub ApplyJournalRef()
    If JQuery Is Nothing Then Set JQuery = New JournalQuery
    Call ApplyIt(JQuery)
End Sub

Sub ApplyRSChanger()
    If RSChangerQuery Is Nothing Then
        Set RSChangerQuery = New ChemRDigitQuery
        Set RSChangerQuery.FontListener = Me
    End If
    Call ApplyIt(RSChangerQuery)
End Sub

Sub FormatNMRSpec()
    Dim Done As Boolean
    
    Done = FormatAsNMRSpec
    If Done Then Exit Sub
    
    Done = FormatAsNMRSpecR
End Sub

Function FormatAsNMRSpec() As Boolean
    Dim o As NMRSpecFormQuery
    Set o = New NMRSpecFormQuery
    o.SpecFormat = GetDocProperty(NMRSpecFormatPropName, NMRSpecFormatDefault)
    o.TableFormat = GetDocProperty(NMRTableFormatPropName, o.SpecFormat)
    o.JValueStyle = GetDocProperty(JValuesFormatPropName, JValuesFormaDefault)
    FormatAsNMRSpec = ApplyIt(o, False)
End Function

Function FormatAsNMRSpecR() As Boolean
    Dim DefSpecFormat As String
    DefSpecFormat = GetDocProperty(NMRSpecFormatPropName, NMRSpecFormatDefault)
    Dim o As NMRSpecFormRQuery
    Set o = New NMRSpecFormRQuery
    o.TableFormat = GetDocProperty(NMRTableFormatPropName, o.SpecFormat)
    
    FormatAsNMRSpecR = FormatAsNMRSpecRA(DefSpecFormat, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(NMRSpecFormat_1H_d_J, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(NMRSpecFormat_1H_J_d, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(NMRSpecFormat_d_1H_J, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(NMRSpecFormat_d_J_1H, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(NMRSpecFormat_J_1H_d, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(NMRSpecFormat_J_d_1H, True, o)
    If FormatAsNMRSpecR Then Exit Function
    FormatAsNMRSpecR = FormatAsNMRSpecRA(DefSpecFormat, False, o)
    If FormatAsNMRSpecR Then Exit Function
End Function

Private Function FormatAsNMRSpecRA(ByVal fmt As String, ByVal Strict As Boolean, o As NMRSpecFormRQuery) As Boolean
    o.SpecFormat = fmt
    o.Strict = Strict
    FormatAsNMRSpecRA = ApplyIt(o, False)
End Function

Public Sub ShowNMRSpecSetting()
    Dim frmw As frmNMRSpecFormat
    Set frmw = New frmNMRSpecFormat
    
    frmw.NMRSpecFormat = GetDocProperty(NMRSpecFormatPropName, NMRSpecFormatDefault)
    frmw.JValuesStyle = GetDocProperty(JValuesFormatPropName, JValuesFormaDefault)
    frmw.NMRTableFormat = GetDocProperty(NMRTableFormatPropName, frmw.NMRSpecFormat)

#If VBA = 0 Then
    frmw.Show vbModal
#Else
    frmw.Show
#End If

    If frmw.DialogResult = vbOK Then
        Call SetDocProperty(NMRSpecFormatPropName, frmw.NMRSpecFormat)
        Call SetDocProperty(JValuesFormatPropName, frmw.JValuesStyle)
        Call SetDocProperty(NMRTableFormatPropName, frmw.NMRTableFormat)
    End If
End Sub

''''''''''''''''''''''''''''''''''''''

Public Function GetDocProperty(ByVal PropName As String, ByVal Default As String) As String
    Err.Raise 1, , "Unsupported."
End Function

Public Sub SetDocProperty(ByVal PropName As String, ByVal Value As String)
    Err.Raise 1, , "Unsupported."
End Sub

Public Function Supported(ByVal Command As String) As Boolean
    Select Case Command
    Case _
        "ChangeStyleForR", _
        "ApplyStyleForChemName", _
        "ApplyStyleForChemFormula", _
        "ApplyStyleForJournalReference", _
        "ApplyStyleForIon"
        Supported = True
    Case "ApplyNMRFormat", _
        "ShowNMRFormatSetting"
        Supported = False
    Case Else
        Supported = False
    End Select
End Function
