Attribute VB_Name = "ChemFormatterLib"
' Copyright 2003-2004 by Kazuya Ujihara <ujihara.jp>.
'
' The contents of this file are subject to the Mozilla Public License Version 1.1
' (the "License"); you may not use this file except in compliance with the License.
' You may obtain a copy of the License at http://www.mozilla.org/MPL/
'
' Software distributed under the License is distributed on an "AS IS" basis,
' WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
' for the specific language governing rights and limitations under the License.
'
' The Original Code is 'ChemFormatter'.
'
' The Initial Developer of the Original Code is Kazuya Ujihara. Portions created by
' the Initial Developer are Copyright (C) 2003-2004 by Kazuya Ujihara.
' All Rights Reserved.
'
' Contributor(s): all the names of the contributors are added in the source code
' where applicable.

Option Explicit

Public Const LITERAL_INDICATOR = "="

#If WIN <> 0 Then

Dim fso As Object 'FileSystemObject

#End If

Public ItalicPrefixes As String
Public Const DefaultItalicPrefixes = "syn|anti|meso|racemi|cis|trans|rel|l|d|dl|i|endo|exo|sec|tert|n|s|t|o|m|p|vic|gem|cisoid|transoid|r|t|c|ent|gache|erythro|threo"
Public Const AllAtoms = "H|He|Li|Be|B|C|N|O|F|Ne|Na|Mg|Al|Si|P|S|Cl|Ar|K|Ca|Sc|Ti|V|Cr|Mn|Fe|Co|Ni|Cu|Zn|Ga|Ge|As|Se|Br|Kr|Rb|Sr|Y|Zr|Nb|Mo|Tc|Ru|Rh|Pd|Ag|Cd|In|Sn|Sb|Te|I|Xe|Cs|Ba|La|Ce|Pr|Nd|Pm|Sm|Eu|Gd|Tb|Dy|Ho|Er|Tm|Yb|Lu|Hf|Ta|W|Re|Os|Ir|Pt|Au|Hg|Tl|Pb|Bi|Po|At|Rn|Fr|Ra|Ac|Th|Pa|U|Np|Pu|Am|Cm|Bk|Cf|Es|Fm|Md|No|Lr"

Public Const ckqModeOptionDefault = 0
Public Const ckqModeOptionIncludeApostrophe = 1

Public Function GetChemSubscriptQuery() As ChemSubscriptQuery
    Static a As ChemSubscriptQuery
    If a Is Nothing Then
        Set a = New ChemSubscriptQuery
    End If
    Set GetChemSubscriptQuery = a
End Function

Public Function GetChemSuperscriptQuery() As ChemSuperscriptQuery
    Static a As ChemSuperscriptQuery
    If a Is Nothing Then
        Set a = New ChemSuperscriptQuery
    End If
    Set GetChemSuperscriptQuery = a
End Function

Public Function GetTripleBondTransQuery() As TripleBondTransQuery
    Static a As TripleBondTransQuery
    If a Is Nothing Then
        Set a = New TripleBondTransQuery
    End If
    Set GetTripleBondTransQuery = a
End Function

Public Function GetChemItalicPrefixesQuery() As ChemKeywordsQuery
    Static a As ChemKeywordsQuery
    If a Is Nothing Then
        If ItalicPrefixes = "" Then
#If WIN <> 0 Then
            ItalicPrefixes = GetFileLFTextAsRE("prefixes.txt", DefaultItalicPrefixes)
#Else
            ItalicPrefixes = DefaultItalicPrefixes
#End If
        End If
        Set a = New ChemKeywordsQuery
        a.Keywords = ItalicPrefixes
    End If
    Set GetChemItalicPrefixesQuery = a
End Function

#If WIN <> 0 Then

Function GetFileLFTextAsRE(ByVal FileName As String, ByVal Default As String) As String
    If fso Is Nothing Then Set fso = CreateObject("Scripting.FileSystemObject")

    Dim FolderPath As String
    Dim Path As String

    FolderPath = GetSpecialFolderName(CSIDL_APPDATA, True)
    Path = FolderPath & "\Ujihara\ChemFormatter\" & FileName
    If fso.FileExists(Path) Then GoTo 100
    
    FolderPath = GetSpecialFolderName(CSIDL_COMMON_APPDATA, True)
    Path = FolderPath & "\Ujihara\ChemFormatter\" & FileName
    If fso.FileExists(Path) Then GoTo 100

#If VBA = 0 Then
    Path = App.Path & "\" & FileName
    If fso.FileExists(Path) Then GoTo 100
#End If
    GetFileLFTextAsRE = Default
    Exit Function
100:
    GetFileLFTextAsRE = GetFPFileLFTextAsRE(Path)
End Function

Function GetFPFileLFTextAsRE(ByVal Path As String) As String
    If fso Is Nothing Then Set fso = CreateObject("Scripting.FileSystemObject")

    Dim ips As String
    ips = ""
    Dim ts As Object 'TextStream
    Set ts = fso.OpenTextFile(Path, 1, False, 0) 'ForReading=1:TristateFalse=0
    While Not ts.AtEndOfLine
        Dim s As String
        s = Trim$(ts.ReadLine)
        If s <> "" Then
            ips = ips & s & "|"
        End If
    Wend
    ips = Left$(ips, Len(ips) - 1)
    ts.Close
    GetFPFileLFTextAsRE = ips
End Function

#End If

Public Function GetChemStereoQuery() As ChemKeywordsQuery
    Static a As ChemKeywordsQuery
    If a Is Nothing Then
        Set a = New ChemKeywordsQuery
        a.Keywords = "E|Z|EZ|ZE|R|S|RS|SR"
    End If
    Set GetChemStereoQuery = a
End Function

Public Function GetChemAtomQuery() As ChemKeywordsQuery
    Static a As ChemKeywordsQuery
    If a Is Nothing Then
        Set a = New ChemKeywordsQuery
        a.Keywords = AllAtoms
        a.ModeOption = ckqModeOptionIncludeApostrophe
    End If
    Set GetChemAtomQuery = a
End Function

Public Function GetChemSmallQuery() As ChemKeywordsQuery
    Static a As ChemKeywordsQuery
    If a Is Nothing Then
        Set a = New ChemKeywordsQuery
        a.Keywords = "D|L|DL"
    End If
    Set GetChemSmallQuery = a
End Function

Function MakeCommand(ByVal Command As String) As String
    MakeCommand = Command & vbLf
End Function

Function MakeCommand1(ByVal Command As String, ByVal Parm1 As String) As String
    MakeCommand1 = MakePushCommand(Parm1) & MakeCommand(Command)
End Function

Function MakeCommand2(ByVal Command As String, ByVal Parm1 As String, ByVal Parm2 As String) As String
    MakeCommand2 = MakePushCommand(Parm1) & MakePushCommand(Parm2) & MakeCommand(Command)
End Function

Function MakePushCommand(ByVal Param As String)
    MakePushCommand = LITERAL_INDICATOR & Param & vbLf
End Function

Function SplitNMRSpecFormat(ByVal fmt As String, ByRef Orders() As Long, ByRef OrdersOfPros() As Long)
    Dim i As Long
    ReDim Orders(NMRSpecFormatter_NumberOfValues - 1) As Long
    ReDim OrdersOfPros(NMRSpecFormatter_NumberOfValues - 1) As Long
    For i = 0 To NMRSpecFormatter_NumberOfValues - 1
        Orders(i) = -1
        OrdersOfPros(i) = -1
    Next
    Orders(NMRSpecFormatterC_ChemicalShift) = 0
    OrdersOfPros(0) = NMRSpecFormatterC_ChemicalShift
    Orders(NMRSpecFormatterC_Comments) = 4
    OrdersOfPros(4) = NMRSpecFormatterC_Comments
    
    Dim ss() As String
    Call SplitWord(fmt, ",", ss)
    If UBound(ss) <> 2 Then GoTo 2000
    For i = 0 To 2
        Dim s As String
        s = Trim(LCase(ss(i)))
        Select Case s
        Case "1h"
            Orders(i + 1) = NMRSpecFormatterC_NumberAndAtom
            OrdersOfPros(NMRSpecFormatterC_NumberAndAtom) = i + 1
        Case "d"
            Orders(i + 1) = NMRSpecFormatterC_Pattern
            OrdersOfPros(NMRSpecFormatterC_Pattern) = i + 1
        Case "j"
            Orders(i + 1) = NMRSpecFormatterC_JValues
            OrdersOfPros(NMRSpecFormatterC_JValues) = i + 1
        Case Else
            '
        End Select
    Next
    For i = 0 To NMRSpecFormatter_NumberOfValues - 1
        If Orders(i) = -1 Then GoTo 2000
    Next
    Exit Function
2000:
    Orders(1) = NMRSpecFormatterC_NumberAndAtom
    OrdersOfPros(NMRSpecFormatterC_NumberAndAtom) = 1
    
    Orders(2) = NMRSpecFormatterC_Pattern
    OrdersOfPros(NMRSpecFormatterC_Pattern) = 2
    
    Orders(3) = NMRSpecFormatterC_JValues
    OrdersOfPros(NMRSpecFormatterC_JValues) = 3
End Function

Function MargeNMRSpecFormat(PropOrders() As Long)
    MargeNMRSpecFormat = ""
    Dim i As Long
    For i = 1 To NMRSpecFormatter_NumberOfValues - 1
        Dim ID As Long
        ID = PropOrders(i)
        Select Case ID
        Case NMRSpecFormatterC_NumberAndAtom
            MargeNMRSpecFormat = MargeNMRSpecFormat & "1H" & ", "
        Case NMRSpecFormatterC_Pattern
            MargeNMRSpecFormat = MargeNMRSpecFormat & "d" & ", "
        Case NMRSpecFormatterC_JValues
            MargeNMRSpecFormat = MargeNMRSpecFormat & "J" & ", "
        End Select
    Next
    MargeNMRSpecFormat = Left$(MargeNMRSpecFormat, Len(MargeNMRSpecFormat) - 2)
End Function


