VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "NMRSpecFormRQuery"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' Copyright 2004 by Kazuya Ujihara <ujihara.jp>.
'
' The contents of this file are subject to the Mozilla Public License Version 1.1
' (the "License"); you may not use this file except in compliance with the License.
' You may obtain a copy of the License at http://www.mozilla.org/MPL/
'
' Software distributed under the License is distributed on an "AS IS" basis,
' WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
' for the specific language governing rights and limitations under the License.
'
' The Original Code is 'ChemFormatter'.
'
' The Initial Developer of the Original Code is Kazuya Ujihara. Portions created by
' the Initial Developer are Copyright (C) 2004 by Kazuya Ujihara.
' All Rights Reserved.
'
' Contributor(s): all the names of the contributors are added in the source code
' where applicable.

'2004-06-03 New release
'2004-06-24 Fixed infinite loop bugs

Option Explicit

Private Const UNIQUE_EndOfSpectrum = "#EndOfSpectrum#"

Dim c As String
Dim ptr As Long
Dim Source As String

Dim PropOrders() As Long
Dim OrdersOfPros() As Long
Dim TablePropOrders() As Long
Dim TableOrdersOfPros() As Long

Public Strict As Boolean

Public Property Let SpecFormat(ByVal fmt As String)
    Call SplitNMRSpecFormat(fmt, PropOrders, OrdersOfPros)
End Property

Public Property Get SpecFormat() As String
    SpecFormat = MargeNMRSpecFormat(PropOrders)
End Property

Public Property Let TableFormat(ByVal fmt As String)
    Call SplitNMRSpecFormat(fmt, TablePropOrders, TableOrdersOfPros)
End Property

Public Function GetCommands(ByVal Text As String) As String
    Dim i As Long
    Dim s As String 'temp string var
    Source = NormalizeString(Text)
    
    On Error GoTo 1000
    
    Dim TopSpaceNum As Long
    
    ptr = InStr(1, Source, ChrW$(&H3B4), vbTextCompare) 'Chrw$(&H3B4) is small delta
    If ptr = 0 Then
        ptr = 1
    Else
        ptr = ptr + 1
    End If
    
    Call GetCC
    Call SkipSpace
    TopSpaceNum = ptr - 2
    
    GetCommands = GetCommands & MakeCommand1("GoTo", UNIQUE_EndOfSpectrum)
    GetCommands = GetCommands & MakeCommand("TypeParagraph")
    GetCommands = GetCommands & MakeCommand1("TypeText", "!")
    GetCommands = GetCommands & MakeCommand1("GoTo", "PrevChar")
    
    Dim NumberOfPeaks As Long: NumberOfPeaks = 0
    Do
        Dim IQs(NMRSpecFormatter_NumberOfValues - 1) As String
        Dim RawIQs(NMRSpecFormatter_NumberOfValues - 1) As String
        For i = 0 To NMRSpecFormatter_NumberOfValues - 1
            IQs(i) = ""
            RawIQs(i) = ""
        Next
        
        'read chemical shift
        
        Call SkipSpace
        s = EatChemicalShift
        RawIQs(NMRSpecFormatterC_ChemicalShift) = s
        IQs(NMRSpecFormatterC_ChemicalShift) = MakeCommand1("TypeText", s)
        
        NumberOfPeaks = NumberOfPeaks + 1
        
        'read first paranthesis
        
        Call SkipSpace
        If c <> "(" Then GoTo GoReadFinished
        Call GetCC
        Call SkipSpace
        
        For i = 1 To NMRSpecFormatter_NumberOfValues - 2
            Call SkipSpace
            Dim v As Boolean
            Select Case PropOrders(i)
            Case NMRSpecFormatterC_NumberAndAtom
                s = EatNumberOfAtom(v)
            Case NMRSpecFormatterC_Pattern
                s = EatCouplingPattern(v)
            Case NMRSpecFormatterC_JValues
                s = EatJValues(v)
            Case Else
#If OLDVB = 0 Then
                Debug.Assert False
#End If
            End Select
            RawIQs(PropOrders(i)) = s
            IQs(PropOrders(i)) = MakeCommand1("TypeText", s)
            
            If v Then
                Call SkipSpace
                Select Case c
                Case ")"
                    Call GetCC
                    GoTo GoReadFinished
                Case ","
                    Call GetCC
                Case Else
                    Err.Raise 1, , "Missing close parenthesis."
                End Select
            End If
        Next

        'read comments
GoReadComment:
        Dim CommentStart As Long: CommentStart = 0
        Dim CommentEnd As Long: CommentEnd = 0
        Call SkipSpace
        If c = ")" Then
            Call GetCC
            GoTo GoReadFinished
        End If
        CommentStart = ptr - 1
        Call EatUntilClosePar
        CommentEnd = ptr - 1
        Call GetCC
        RawIQs(NMRSpecFormatterC_Comments) = "###DUMMY###"
        IQs(NMRSpecFormatterC_Comments) = _
            MakeCommand2("Copy", CommentStart - 1, CommentEnd - CommentStart) & _
            MakeCommand("Paste")

GoReadFinished:
        'check reqired props
        If Strict Then
            If RawIQs(NMRSpecFormatterC_ChemicalShift) = "" Then Err.Raise 1, , "Missing chemical shift."
            If RawIQs(NMRSpecFormatterC_JValues) <> "" Then
                If RawIQs(NMRSpecFormatterC_Pattern) = "" Then
                    Err.Raise 1, , "Missing coupling pattern."
                End If
            End If
            If RawIQs(NMRSpecFormatterC_NumberAndAtom) = "" Then Err.Raise 1, , "Missing number and atom."
        End If
        
        'compile commands
        GetCommands = GetCommands & IQs(NMRSpecFormatterC_ChemicalShift)
        GetCommands = GetCommands & MakeCommand1("TypeText", vbTab)
        For i = 1 To NMRSpecFormatter_NumberOfValues - 2
            GetCommands = GetCommands & IQs(TablePropOrders(i))
            GetCommands = GetCommands & MakeCommand1("TypeText", vbTab)
        Next
        GetCommands = GetCommands & IQs(NMRSpecFormatterC_Comments)
        GetCommands = GetCommands & MakeCommand("TypeParagraph")
        
        'check end
        
        Call SkipSpace
        Select Case c
        Case ".", "", vbCr
            Call GetCC
            Exit Do
        Case ","
            Call GetCC
        Case Else
            Err.Raise 1, , "Missing comma."
        End Select
    Loop
    Do While c = " "
        Call GetCC
    Loop
    
    'erase !
    GetCommands = GetCommands & MakeCommand1("GoTo", "NextChar")
    GetCommands = GetCommands & MakeCommand("TypeBackspace")
    
    Dim EndPointer As Long
    EndPointer = ptr - 2
    GetCommands = GetCommands & MakePushCommand(1 + TopSpaceNum)
    GetCommands = GetCommands & MakePushCommand("$CursorPosition")
    GetCommands = GetCommands & MakePushCommand(1)
    GetCommands = GetCommands & MakeCommand("Minus")
    GetCommands = GetCommands & MakePushCommand(EndPointer)
    GetCommands = GetCommands & MakeCommand("Minus")
    GetCommands = GetCommands & MakeCommand2("Cut", TopSpaceNum, EndPointer - TopSpaceNum)
    GetCommands = ReplaceString(GetCommands, UNIQUE_EndOfSpectrum, EndPointer)
    GetCommands = GetCommands & MakeCommand("Select")
    GetCommands = GetCommands & MakeCommand1("ApplyStyle", "NMR Spec Table")
    If NumberOfPeaks < 1 Then GetCommands = ""
3000:
    On Error GoTo 0
    Exit Function
1000:
    GetCommands = ""
    Resume 3000
End Function

Private Function EatChemicalShift() As String
    Dim SaveC As String, SavePtr As Long
    SaveC = c: SavePtr = ptr
        
    Dim s0 As String, s1 As String
    
    On Error GoTo 1000
    s0 = EatFNum()
    
    Call SkipSpace
    Select Case c
    Case "-", ChrW$(&HFF5E), "~", ChrW$(&H8160)
        Call GetCC
        Call SkipSpace
        s1 = EatFNum()
        Call SkipSpace
        EatChemicalShift = s0 & "-" & s1
    Case Else
        EatChemicalShift = s0
    End Select
2000:
    On Error GoTo 0
    Exit Function
1000:
    c = SaveC: ptr = SavePtr
    EatChemicalShift = ""
    Resume 2000
End Function

Private Function EatNumberOfAtom(ByRef Vaild As Boolean) As String
    Dim SaveC As String, SavePtr As Long
    SaveC = c: SavePtr = ptr
    
    Dim s0 As String, s1 As String
    s0 = EatFPNum()
    If s0 = "" Then GoTo 2000
    s1 = EatWord()
    If s1 = "" Then GoTo 2000
    EatNumberOfAtom = s0 & s1
    Vaild = True
    
    Exit Function
2000:
    EatNumberOfAtom = ""
    Vaild = False
    c = SaveC: ptr = SavePtr
End Function

Private Function EatCouplingPattern(ByRef Vaild As Boolean) As String
    Dim SaveC As String, SavePtr As Long
    SaveC = c: SavePtr = ptr
    Dim cp As Long
    cp = ptr - 1
    
    If c = "J" Then GoTo 2000
    Do
        Select Case c
        Case ""
            Err.Raise 1, , "Unexpected end of line."
        Case ",", ")", ""
            Exit Do
        Case "0" To "9"
            GoTo 2000
        Case Else
            Call GetCC
        End Select
    Loop
    EatCouplingPattern = Mid$(Source, cp, ptr - 1 - cp)
    Vaild = True
    Exit Function
2000:
    EatCouplingPattern = ""
    Vaild = False
    c = SaveC: ptr = SavePtr
End Function

Private Function EatJValues(ByRef Vaild As Boolean) As String
    EatJValues = ""
    
    Dim SaveC As String, SavePtr As Long
    Dim s As String, ss As String
    Dim ErrNumber As Long
    
    SaveC = c
    SavePtr = ptr
    
    s = EatWord()
    If s <> "J" Then GoTo GoNoJValue
    
    Call SkipSpace
    s = EatChar("=")
    If s = "" Then GoTo GoNoJValue
    ss = ""
    Do
        Call SkipSpace
        Dim ThisJValue As String
        On Error Resume Next
        Err.Clear
        ThisJValue = EatFNum()
        ErrNumber = Err.Number
        On Error GoTo 0
        If ErrNumber <> 0 Then
            c = SaveC: ptr = SavePtr
            GoTo GoJValuesEnd
        End If
        Call SkipSpace
        Select Case c
        Case ""
            Err.Raise 1, , "Unexpected end of line."
        Case ")"
            ss = ss & ThisJValue & ", "
            GoTo GoJValuesEnd
        Case ","
            ss = ss & ThisJValue & ", "
            SaveC = c: SavePtr = ptr
            Call GetCC
            GoTo GoContinueLoop
        Case Else
            s = EatWord
            If s = "Hz" Then
                Call SkipSpace
                Select Case c
                Case ")"
                    ss = ss & ThisJValue & ", "
                    GoTo GoJValuesEnd
                Case ","
                    ss = ss & ThisJValue & ", "
                    SaveC = c: SavePtr = ptr
                    Call GetCC
                    GoTo GoContinueLoop
                Case Else
                    '
                End Select
            End If
            c = SaveC: ptr = SavePtr
            GoTo GoJValuesEnd
        End Select
GoContinueLoop:
    Loop
GoJValuesEnd:
    EatJValues = XTrim(Left$(ss, Len(ss) - 2))
    If EatJValues = "" Then GoTo GoNoJValue
    Vaild = True
    Exit Function

GoNoJValue:
    c = SaveC
    ptr = SavePtr
    Vaild = False
    Exit Function
End Function

Private Sub EatUntilClosePar()
    Dim PDepth As Long
    PDepth = 0
    Do
        Select Case c
        Case ""
            Err.Raise 1, , "Unexpected end of line."
        Case ")"
            If PDepth <= 0 Then
                Exit Sub
            Else
                PDepth = PDepth - 1
            End If
        Case "("
            PDepth = PDepth + 1
        End Select
        Call GetCC
    Loop
End Sub

Private Function EatWord() As String
    EatWord = ""
    Do While IsAlpha(c)
        EatWord = EatWord & c
        c = GetCC()
    Loop
End Function

Private Function IsAlpha(ByVal c As String)
    Dim n As Long
    n = AscW(c)
    IsAlpha = (Unicode_BigA <= n And n <= Unicode_BigZ) Or (Unicode_SmallA <= n And n <= Unicode_SmallZ)
End Function

Private Function EatNaturalNum() As String
    EatNaturalNum = ""
    Do While IsDigit(c)
        EatNaturalNum = EatNaturalNum & c
        c = GetCC()
    Loop
End Function

Private Function EatChar(ByVal Find As String) As String
#If OLDVB = 0 Then
    Debug.Assert Len(Find) = 1
#End If
    If c = Find Then
        EatChar = c
        c = GetCC()
    Else
        EatChar = ""
    End If
End Function

Private Sub SkipSpace()
    Do While IsSpace(c)
        c = GetCC()
    Loop
End Sub

Private Function GetCC() As String
    c = Mid$(Source, ptr, 1)
    ptr = ptr + 1
    GetCC = c
End Function

Private Function EatFPNum() As String
    Dim s As String: s = ""
    s = s & EatNaturalNum()
    If s = "" Then Exit Function
    If c = "." Then
        s = s & c
        c = GetCC()
    End If
    s = s & EatNaturalNum()
    EatFPNum = s
End Function

Private Function EatFNum() As String
    Dim s As String: s = ""
    s = s & EatChar("-")
    Call SkipSpace
    Dim ss As String
    ss = EatFPNum()
    If ss = "" Then Err.Raise 1, , "No Digit."
    EatFNum = s & ss
End Function

Private Function IsSpace(ByVal c As String) As Boolean
    Dim n As Long
    n = AscW(c)
    IsSpace = (n = Unicode_Space)
End Function

Private Function IsDigit(ByVal c As String) As Boolean
    Dim n As Long
    n = AscW(c)
    IsDigit = Unicode_0 <= n And n <= Unicode_9
End Function

Private Sub Class_Initialize()
    Me.SpecFormat = ""
    Me.TableFormat = ""
    Me.Strict = False
End Sub



